package org.w3c.rdfpic.core;
import org.w3c.rdfpic.imagedrv.*;
import org.w3c.tools.jpeg.JpegCommentHandler;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.Writer;
import java.io.Reader;
import java.io.File;
import java.io.OutputStream;
import java.io.FileOutputStream;
import javax.swing.ImageIcon;
import java.awt.Image;

import java.net.URL;
/**
 *A class to get and send meta (text) data to image files. (Currently jpeg's only)
 */
public class ImgMetaHandler
{
    URL url;
    ProtocolHandler ph;
    ImgMeta im;
    Image currentImg;

    public ImgMetaHandler()
    {
	url=null;
	im=null;
	ph=null;
	currentImg=null;
    }
    /**
     * Selects (and fully buffers!) an image.
     * @param url The url of the image.  Supports whatever ProtocolHandler does (currently http 1.1 and file protocols)
     */
    public void setURL(URL url) throws PHException,UnknownImgTypeException//gets called to load a new image
    {
	im=null;ph=null;currentImg=null;
	if(url!=null)
	    {
		ph=new ProtocolHandler(url);//loads and buffers data - throws PHException if there are problems
		im=new JpegMeta(ph);//Interprets data, throws UnknownImgTypeException.
		currentImg=(new ImageIcon(ph.read())).getImage();
	    }

	this.url = url;
    }
    /**
     * Returns a (cached) Image of the graphical data of the file currently loaded.
     * Returns <code>null</code> if nothing is loaded.
     */
    public Image getImg()
    {
	return currentImg;
    }
    /**
     * Returns the size of the image file loaded in bytes.
     * @exception PHException when no file is loaded.
     */
    public int getFileSize() throws PHException
    {
	if(im==null) throw new PH_noURL();
	return ph.fileSize();
    }

    /**
     * Returns the active URL.  (Not a copy - so do not change!)
     */
    public URL getURL()
    {
	return url;
    }
    /**
     * Reads the text embedded in the file.
     */
    public String read() throws IOException
    {
	if(im==null) throw new PH_noURL("No URL loaded");
	return im.read();
    }
    /**
     * (Over)writes the text embedded in the file.
     * @param comment The comment to embed.
     */
    public void write(String comment) throws IOException
    {
	if(im==null) throw new PH_noURL("No URL loaded");
	im.write(comment);
    }
    /**
     * Returns whether the image is readable.
     */
    public boolean canRead()
    {
	if(im==null) return false;
	return im.canRead();
    }
    /**
     * Returns whether the image is writable.  Because of limitations in HTTP 1.1, it's impossible to know in advance whether
     * PUT permissions will be granted.  So, if HTTP 1.1 is being used, is simply assumes it can write, but signals an IO error
     * when you actually try to.
     */
    public boolean canWrite()
    {
	if(im==null) return false;
	return im.canWrite();
    }
}
