package org.w3c.rdfpic.core;
import java.io.*;
import java.util.*;
import org.w3c.rdf.model.*;
import org.w3c.rdf.syntax.*;

/**
 * A simple RDFSerializer.
 */

public class RDFSerial implements RDFSerializer {
    private Writer wr;
    private int indent;
    private String indentStr="";
    static final private String INDENT_LEVEL_STR="  ";
    private void put(String str) throws IOException {
	wr.write(indentStr+str);
	indentStr="";
    }
    private void putln(String str) throws IOException {
	wr.write(indentStr+str);
	reindent();
    }
    private void reindent() {
	StringBuffer strbuf=new StringBuffer();
	strbuf.append('\n');
	for(int i=0;i<indent;i++) strbuf.append(INDENT_LEVEL_STR);
	indentStr=strbuf.toString();
    }
    public void serialize(Model model, Writer wr) throws IOException, ModelException {
	if(model==null) return;
	TreeMap nameSpaces = new TreeMap();// key=ns-uri; value=shortcut
	TreeMap abouts = new TreeMap(); // key=subject; value=ArrayList of Statements
	Enumeration stats = model.elements();
	int shortid =0;
	nameSpaces.put("http://www.w3.org/1999/02/22-rdf-syntax-ns#","rdf");
	nameSpaces.put("http://www.w3.org/TR/1999/PR-rdf-schema-19990303#","rdfs");
	this.wr=wr;
	indent=1;
        putln("<?xml version='1.0' encoding='ISO-8859-1'?>");
	indent+=2;
	putln("<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"");
	putln("xmlns:rdfs=\"http://www.w3.org/TR/1999/PR-rdf-schema-19990303#\"");
	
	while(stats.hasMoreElements())
	    {
		Statement current;
		String subj;
		String pred,uri;
		
		current=(Statement)stats.nextElement();
		subj=current.subject().getURI();
		pred=current.predicate().getURI();
		int hashindx=pred.indexOf('#');
		if(hashindx==-1) uri="";
		else {
		    uri=pred.substring(0,hashindx);
		    pred=pred.substring(hashindx+1);
		}
		
		if(!nameSpaces.containsKey(uri)) {
		    nameSpaces.put(uri,"s"+shortid);
		    putln("xmlns:s"+shortid+"=\""+uri+"#\"");//hash for backward compatibility
		    shortid++;
		}
		if(abouts.containsKey(subj)) {
		    ((ArrayList)(abouts.get(subj))).add(current);
		} else {
		    ArrayList arrList = new ArrayList();
		    arrList.add(current);
		    abouts.put(subj,arrList);
		}
	    }
	indentStr="";indent--;
	putln(">");
	
	Iterator subjs = abouts.values().iterator();
	while(subjs.hasNext()) {
	    ArrayList arrList = (ArrayList)subjs.next();
	    indent++;
	    putln("<rdf:Description rdf:about=\""+((Statement)arrList.get(0)).subject().getURI()+"\">");
	    for(int i=0;i<arrList.size();i++) {
		Statement current;
		String pred,uri,val;
		current=(Statement)arrList.get(i);
		pred=current.predicate().getURI();
		int hashindx=pred.indexOf('#');
		if(hashindx==-1) uri="";
		else {
		    uri=pred.substring(0,hashindx);
		    pred=pred.substring(hashindx+1);
		}
		val=current.object().getLabel();
		String nsshort = (String)nameSpaces.get(uri);
		putln("<"+nsshort+':'+pred+'>'+val+"</"+nsshort+':'+pred+'>');
	    }
	    indent--;reindent();
	    putln("</rdf:Description>");
	}
	indent--;reindent();
	putln("</rdf:RDF>");
	wr.write("\n"); wr.flush();
    }
}
