package org.w3c.rdfpic.ui;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JComboBox;
import javax.swing.ComboBoxModel;
import java.util.Vector;
import java.awt.Dimension;

/**
 * MemComboBox is a JComboBox that will add any new items selected automatically to the list of items contained.
 * This is useful for a history, so that it is not necessary to manually add any items, or for an editable combo-
 * box, in which all of the users input is memorized in case of later recall.
 */

public class MemComboBox extends JComboBox
{
    private void startupCode()
    {
	addActionListener(new ActionListener (){
	    public void actionPerformed(ActionEvent e) {
		MemComboBox cb = (MemComboBox)e.getSource();
		Object item = cb.getSelectedItem();
		if (item instanceof String)
		    {
			String val = (String)item;
			if (val != null && val.trim().length() > 0 && !cb.hasItem(val))
			    cb.insertItemAt(val,0);
		    }
		else if(item != null && !cb.hasItem(item))
		    cb.insertItemAt(item,0);
	    }
	});
        Dimension sz = getPreferredSize();
        setPreferredSize(new Dimension(256,(int)sz.getHeight()));
    }
    public MemComboBox() {super();startupCode();}
    public MemComboBox(String[] strArr) {super(strArr);startupCode();}
    public MemComboBox(Vector vec){super(vec);startupCode();}
    public MemComboBox(ComboBoxModel model){super(model);startupCode();}

    public void setSelectedItem(Object item)
    {
	if (item!=null && !hasItem(item) &&  !item.equals("") )
	    insertItemAt(item,0);
	super.setSelectedItem(item);
    }
    
    boolean hasItem(Object obj) {
	for (int i=0; i < getItemCount(); ++i) {
	    if (getItemAt(i).equals(obj)) {
		return true;
	    }
	}
	return false;
    }
}
